#include "debug.h"
#include "math.h"

uint16_t ADC_Buffer[64];
uint8_t volatile adc_flag = 0;

//定时器TIM3初始化
void TIM3_Init(uint16_t period)
{
    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    RCC_ClocksTypeDef clocks;

    RCC_GetClocksFreq(&clocks);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

    TIM_TimeBaseInitStructure.TIM_Period = period; //设置0.2ms采样一次
    TIM_TimeBaseInitStructure.TIM_Prescaler = clocks.PCLK1_Frequency / 1000000 - 1;
    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;     //时钟分频，TIM3是通用定时器，基本定时器不用设置
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up; //向上扫描
    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseInitStructure);

    TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update); //选择TRGO作为触发源为定时器更新时间

    //    TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);
    TIM_ClearITPendingBit(TIM3, TIM_IT_Update); //清中断以免一启用中断后立即产生中断

    NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    TIM_Cmd(TIM3, ENABLE); //使能定时中断
}

// ADC1的IN1通道初始化
void ADC1_IN1_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    ADC_InitTypeDef ADC_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_ADC1, ENABLE); //使能GPIOA时钟和ADC

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;     // PA1对应ADC通道1
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN; // GPIO模式为模拟输入
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    RCC_ADCCLKConfig(RCC_PCLK2_Div8); //设置ADC时钟分频为6分频

    ADC_DeInit(ADC1);
    ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                     //配置ADC为独立模式
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;                          //多通道模式下开启扫描模式
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;                    //设置开启连续转换模式
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T3_TRGO; //定时器3外部触发
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;                 //设置ADC数据右对齐
    ADC_InitStructure.ADC_NbrOfChannel = 1;                                //规则转换的ADC通道的数目
    ADC_Init(ADC1, &ADC_InitStructure);                                    //根据ADC_InitStructure中指定的参数初始化ADC1寄存器

    ADC_Cmd(ADC1, ENABLE); //使能ADC1

    ADC_ExternalTrigConvCmd(ADC1, ENABLE); //设置外部触发模式使能

    ADC_ResetCalibration(ADC1); //重置ADC1校准寄存器。

    while (ADC_GetResetCalibrationStatus(ADC1))
        ; //等待复位校准结束

    ADC_StartCalibration(ADC1); //开启AD校准

    while (ADC_GetCalibrationStatus(ADC1))
        ; //等待校准结束
}

// ADC DMA模式配置
void DMA_Tx_Init(void)
{
    DMA_InitTypeDef DMA_InitStructure = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE); //使能开启DMA时钟

    DMA_DeInit(DMA1_Channel1); //复位DMA控制器

    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&ADC1->RDATAR;                 //配置外设地址为ADC数据寄存器地址
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)ADC_Buffer;                        //配置存储器地址为读取ADC值地址
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                             //配置数据源为外设，即DMA传输方式为外设到存储器
    DMA_InitStructure.DMA_BufferSize = sizeof(ADC_Buffer) / sizeof(ADC_Buffer[0]); //设置DMA数据缓冲区大小，此处设置为length
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;               //设置DMA外设递增模式关闭
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                        //设置DMA存储器递增模式开启
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;    //设置外设数据大小为半字，即两个字节
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;            //设置存储器数据大小为半字，即两个字节
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;                                //设置DMA模式为循环传输模式
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;                            //设置DMA传输通道优先级为高，当使用一 DMA通道时，优先级设置不影响
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                   //因为此DMA传输方式为外设到存储器，因此禁用存储器到存储器传输方式
    DMA_Init(DMA1_Channel1, &DMA_InitStructure);                                   //初始化DMA

    DMA_ITConfig(DMA1_Channel1, DMA_IT_TC, ENABLE); //使能传输完成中断

    NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    DMA_Cmd(DMA1_Channel1, ENABLE); //使能DMA
}

//中断处理函数
__attribute__((interrupt())) void DMA1_Channel1_IRQHandler(void)
{
    printf("\r\nEnter DMA interrupt\r\n");
    if (DMA_GetITStatus(DMA1_IT_TC1) != RESET) {
        //中断处理代码
        //        printf("/* ");
        //        for(int i=0;i<64;i++){
        //            printf("%d, ",ADC_Buffer[i]);
        //        }
        //        printf("*/\r\n");
        //        Delay_Ms(50);
        TIM_Cmd(TIM3, DISABLE);
        //        ADC_DMACmd(ADC1, DISABLE);
        //        DMA_Cmd(DMA1_Channel1, DISABLE);
        adc_flag = 1;
        DMA_ClearITPendingBit(DMA1_IT_TC1);
    }
}

__attribute__((interrupt())) void TIM3_IRQHandler()
{
    if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET) {
        printf("\r\nEnter TIM3 interrupt\r\n");
    }
    TIM_ClearITPendingBit(TIM3, TIM_IT_Update); //清中断以免一启用中断后立即产生中断
}

float get_ADC_average(const uint16_t *psrc, uint32_t number)
{
    float sum = 0;

    for (int i = 0; i < number; i++) {
        sum += psrc[i];
    }

    return sum / number;
}

float get_rms_value(uint16_t *psrc, uint32_t number)
{
    float sum = 0;
    float ave = get_ADC_average(psrc, number);

    for (int i = 0; i < number; i++) {
        sum += ((float)psrc[i] - ave) * (psrc[i] - ave);
    }

    ave = sum / number;

    return sqrtf(ave);
}

int main(void)
{
    RCC_ClocksTypeDef clocks;
    GPIO_InitTypeDef gpio_init;

    Delay_Init();
    USART_Printf_Init(115200);
    SystemCoreClockUpdate();

    gpio_init.GPIO_Pin = GPIO_Pin_1;
    gpio_init.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &gpio_init);

    ADC1_IN1_Init();
    DMA_Tx_Init();
    TIM3_Init(625);

    ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_239Cycles5);
    //    TIM_Cmd(TIM3, ENABLE);

    RCC_GetClocksFreq(&clocks);

    printf("系统时钟频率: %dHz\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟频率: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2总线时钟频率: %dHz\r\n", clocks.PCLK2_Frequency);
    printf("ADC采样时钟频率: %dHz\r\n", clocks.ADCCLK_Frequency);

    ADC_DMACmd(ADC1, ENABLE);
    while (1) {
        if (adc_flag == 1) {
            printf("/* ");
            for (int i = 0; i < sizeof(ADC_Buffer) / sizeof(ADC_Buffer[0]); i++) {
                printf("%d, ", ADC_Buffer[i]);
            }
            printf("*/\r\n");
            printf("DC: %.4f\r\n", get_ADC_average(ADC_Buffer, 64) * 3.3 / 4096);
            printf("AC: %.4f\r\n", get_rms_value(ADC_Buffer, 64) * 3.3 / 4096);

            adc_flag = 0;
            TIM_Cmd(TIM3, ENABLE);
            //        DMA_Cmd(DMA1_Channel1, ENABLE);
        }
    }
}
