#include "debug.h"

int16_t calibration_val;

void ADC_Function_Init(void)
{
    ADC_InitTypeDef ADC_InitStructure={0};
    GPIO_InitTypeDef GPIO_InitStructure={0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE );
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE );
    RCC_ADCCLKConfig(RCC_PCLK2_Div8);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    ADC_DeInit(ADC1);
    ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel = 1;
    ADC_Init(ADC1, &ADC_InitStructure);
    ADC_Init(ADC2, &ADC_InitStructure);

    ADC_Cmd(ADC1, ENABLE);
//    ADC_Cmd(ADC2,ENABLE);

    ADC_BufferCmd(ADC1, DISABLE);   //disable buffer
    ADC_ResetCalibration(ADC1);
    while(ADC_GetResetCalibrationStatus(ADC1));
    ADC_StartCalibration(ADC1);
    while(ADC_GetCalibrationStatus(ADC1));
    calibration_val = Get_CalibrationValue(ADC1);

    ADC_BufferCmd(ADC1, ENABLE);   //enable buffer
//    ADC_BufferCmd(ADC2, ENABLE);

//    ADC_TempSensorVrefintCmd(ENABLE);
}

u16 Get_ADC_Val(u8 ch)
{
    u16 val;

    ADC_RegularChannelConfig(ADC1, ch, 1, ADC_SampleTime_239Cycles5 );
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);

    while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC ));

    val = ADC_GetConversionValue(ADC1);

    return val;
}

u16 Get_ADC_Average(u8 ch,u8 times)
{
    u32 temp_val=0;
    u8 t;
    u16 val;

    for(t=0;t<times;t++)
    {
        temp_val+=Get_ADC_Val(ch);
//        Delay_Ms(5);
    }

    val = temp_val/times;

    return val;
}

u16 Get_ConversionVal(s16 val)
{
    if((val+calibration_val)<0) return 0;
    if((calibration_val+val)>4095||val==4095) return 4095;
    return (val+calibration_val);
}
/**
 * @brief 使用内部DAC产生一个直流电压
 * @param amplitude[in] 电压值对应的DAC值
 */
void DAC_Output_DC_Voltage(uint16_t amplitude)
{
    GPIO_InitTypeDef gpio_init = {0};
    DAC_InitTypeDef dac_init = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);

    GPIO_StructInit(&gpio_init);
    gpio_init.GPIO_Pin = GPIO_Pin_4;
    gpio_init.GPIO_Mode = GPIO_Mode_AIN;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &gpio_init);

    DAC_StructInit(&dac_init);
    dac_init.DAC_Trigger = DAC_Trigger_None;
    dac_init.DAC_WaveGeneration = DAC_WaveGeneration_None;
    dac_init.DAC_OutputBuffer = DAC_OutputBuffer_Disable;
    DAC_Init(DAC_Channel_1, &dac_init);

    DAC_Cmd(DAC_Channel_1, ENABLE);

    DAC_SetChannel1Data(DAC_Align_12b_R, amplitude&0xFFF);

    printf("DAC通道1在PA.4引脚输出%.2f伏的直流电压\r\n",DAC_GetDataOutputValue(DAC_Channel_1)*3.3/4096);
}

int main(void)
{
    GPIO_InitTypeDef gpio_init;

    uint16_t ADC_val;
    RCC_ClocksTypeDef clocks;

    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    ADC_Function_Init();
    RCC_GetClocksFreq(&clocks);

    printf("系统时钟：%d\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟：%d\r\n", clocks.PCLK1_Frequency);
    printf("ABP2总线时钟：%d\r\n", clocks.PCLK2_Frequency);
    printf("ADC模块时钟：%d\r\n", clocks.ADCCLK_Frequency);

    printf("ADC校准值：%d\r\n",calibration_val);

    DAC_Output_DC_Voltage(1024);

    gpio_init.GPIO_Pin = GPIO_Pin_0;
    gpio_init.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA,&gpio_init);

    while (1)
    {
        GPIO_WriteBit(GPIOA, GPIO_Pin_0, Bit_RESET);
        ADC_val = Get_ADC_Average(ADC_Channel_1, 1);
        ADC_val = Get_ConversionVal(ADC_val);
        GPIO_WriteBit(GPIOA, GPIO_Pin_0, Bit_SET);

//        printf("ADC：%04d, voltage: %.2f V\r",ADC_val, ADC_val*3.3/4096);
        printf("/* %d */\r\n",ADC_val);
        fflush(stdout);
//        Delay_Ms(5);
    }
}
