#include "debug.h"

static volatile uint8_t WaveType = 0;

//不添加volatile关键字，若使用switch判断，在第2次进入0时会无波形输出
// static uint8_t WaveType = 0;

// 64点幅值为3.3V的正弦波数据值
uint16_t DAC_Value[] = {2048, 2248, 2447, 2642, 2831, 3013, 3185, 3347, 3496, 3631, 3750, 3854, 3940, 4007, 4056, 4086,
                        4095, 4086, 4056, 4007, 3940, 3854, 3750, 3631, 3496, 3347, 3185, 3013, 2831, 2642, 2447, 2248,
                        2048, 1847, 1648, 1453, 1264, 1082, 910, 748, 599, 464, 345, 241, 155, 88, 39, 9,
                        0, 9, 39, 88, 155, 241, 345, 464, 599, 748, 910, 1082, 1264, 1453, 1648, 1847
                       };

void DAC1_Init(void)
{
    GPIO_InitTypeDef gpio_init = {0};
    DAC_InitTypeDef dac_init = {0};

    // 开启GPIOA时钟，因为DAC通道1从PA.4输出
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    // 开启DAC时钟
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);

    gpio_init.GPIO_Pin = GPIO_Pin_4;
    gpio_init.GPIO_Mode = GPIO_Mode_AIN;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;
    // 将PA.4配置为模量输入，ADC和DAC复用时引脚配置为模拟量输入
    GPIO_Init(GPIOA, &gpio_init);

    // DAC配置为无触发模式，也就是往数据保持寄存器(xDHRx)中写入值就对应输出
    dac_init.DAC_Trigger = DAC_Trigger_None;
    // 关闭输出缓存，若开启会减小输出阻抗增大驱动能力
    dac_init.DAC_OutputBuffer = DAC_OutputBuffer_Disable;
    // 不使用波形生成器,DAC可以产生三角波和噪声
    dac_init.DAC_WaveGeneration = DAC_WaveGeneration_None;
    // 如果使用噪声或三角波其幅值设为0.2V
    dac_init.DAC_LFSRUnmask_TriangleAmplitude = DAC_TriangleAmplitude_255;
    DAC_Init(DAC_Channel_1, &dac_init);

    DAC_Cmd(DAC_Channel_1, ENABLE);

    printf("DAC初始化完成...\r\n");
}

/**
 * @brief 修改DAC内部产后的波形幅值
 *
 * @param channel DAC通道
 *        DAC_Channel_1 - DAC Channel1 selected
 *        DAC_Channel_2 - DAC Channel2 selected
 * @param amplitude 幅值设定
 *        DAC_TriangleAmplitude_1
 *        DAC_TriangleAmplitude_3
 *        DAC_TriangleAmplitude_7
 *        DAC_TriangleAmplitude_15
 *        DAC_TriangleAmplitude_31
 *        DAC_TriangleAmplitude_63
 *        DAC_TriangleAmplitude_127
 *        DAC_TriangleAmplitude_255
 *        DAC_TriangleAmplitude_511
 *        DAC_TriangleAmplitude_1023
 *        DAC_TriangleAmplitude_2047
 *        DAC_TriangleAmplitude_4095
 *
 */
static void SetWaveGenerationAmplitude(uint32_t channel, uint32_t amplitude)
{
    uint32_t temp = DAC->CTLR;
    temp &= ~(0xF << 24 | 0xF << 8);
    temp |= amplitude << channel;
    DAC->CTLR = temp;
}

/**
 * @brief 设定DAC触发方式
 *
 * @param channel DAC通道
 *        DAC_Channel_1 - DAC Channel1 selected
 *        DAC_Channel_2 - DAC Channel2 selected
 * @param trigger 触发方式
 *        DAC_Trigger_None - 无触发写入xHDRx后直接输出
 *        DAC_Trigger_T6_TRGO - TIM6定时器触发
 *        DAC_Trigger_T8_TRGO - TIM8定时器触发
 *        DAC_Trigger_T7_TRGO - TIM7定时器触发
 *        DAC_Trigger_T5_TRGO - TIM5定时器触发
 *        DAC_Trigger_T2_TRGO - TIM2定时器触发
 *        DAC_Trigger_T4_TRGO - TIM4定时器触发
 *        DAC_Trigger_Ext_IT9 - 外部中断9引脚触发
 *        DAC_Trigger_Software - 软件触发
 */
static void DAC_SetChannelTrigger(uint32_t channel, uint32_t trigger)
{
    //    DAC_Cmd(DAC_Channel_1, DISABLE);
    DAC->CTLR &= ~(0x3c << channel);
    DAC->CTLR |= trigger << channel;
    //    DAC_Cmd(DAC_Channel_1, ENABLE);
}

static void PA0_INT_KEY_Init(void)
{
    GPIO_InitTypeDef gpio = {0};
    EXTI_InitTypeDef exti = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

    gpio.GPIO_Pin = GPIO_Pin_0;
    gpio.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &gpio);

    GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);

    exti.EXTI_Line = EXTI_Line0;
    exti.EXTI_Mode = EXTI_Mode_Interrupt;
    exti.EXTI_Trigger = EXTI_Trigger_Falling;
    exti.EXTI_LineCmd = ENABLE;
    EXTI_Init(&exti);

    NVIC_EnableIRQ(EXTI0_IRQn);
}

void TIM6_Frequency_Init(uint8_t frequency)
{
    TIM_TimeBaseInitTypeDef tim_init;

    uint32_t temp = frequency * 64 / 2;
    RCC_ClocksTypeDef clocks;

    RCC_GetClocksFreq(&clocks);

    float div = clocks.PCLK1_Frequency / temp;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);

    if (div < 0xffff) {
        tim_init.TIM_Period = (uint16_t)div - 1;
        tim_init.TIM_Prescaler = 0;
    } else {
        if (div / 10 < 0xffff) {
            tim_init.TIM_Prescaler = 10 - 1;
        } else if (div / 100 < 0xffff) {
            tim_init.TIM_Prescaler = 100 - 1;
        } else if (div / 1000 < 0xffff) {
            tim_init.TIM_Prescaler = 1000 - 1;
        }
        tim_init.TIM_Period = (uint16_t)(div / (tim_init.TIM_Prescaler + 1));
    }
    //        tim_init.TIM_ClockDivision = TIM_CKD_DIV1;
    //        tim_init.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM6, &tim_init);

    TIM_SelectOutputTrigger(TIM6, TIM_TRGOSource_Update);

    TIM_ITConfig(TIM6, TIM_IT_Update, ENABLE);
    NVIC_EnableIRQ(TIM6_IRQn);
}

__attribute__((interrupt())) void TIM6_IRQHandler(void)
{
    GPIO_WriteBit(GPIOA, GPIO_Pin_1, !GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1));
    TIM_ClearITPendingBit(TIM6, TIM_IT_Update);
}

__attribute__((interrupt("WCH-Interrupt-fast"))) void EXTI0_IRQHandler(void)
{
    EXTI_ClearITPendingBit(EXTI_Line0);
    Delay_Ms(50);

    if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0) == Bit_RESET) {
        WaveType++;
        switch (WaveType) {
            case 1:
                printf("利用内部波形发生器产生幅值为0.8V的三角波\r\n");
                DAC_SetChannel1Data(DAC_Align_12b_R, 0);
                SetWaveGenerationAmplitude(DAC_Channel_1, DAC_TriangleAmplitude_1023);
                DAC_SetChannelTrigger(DAC_Channel_1, DAC_Trigger_Software);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, DISABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, ENABLE);
                break;
            case 2:
                printf("利用内部波形发生器产生幅值为1.5v的噪声波形\r\n");
                DAC_SetChannel1Data(DAC_Align_12b_R, 0);
                SetWaveGenerationAmplitude(DAC_Channel_1, DAC_TriangleAmplitude_2047);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, ENABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, DISABLE);
                break;
            case 3:
                printf("50Hz幅值为1.5v的正弦波叠加幅值为1.5v的三角波\r\n");
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, DISABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, ENABLE);
                break;
            case 4:
                printf("50Hz幅值为1.5v的正弦波叠加幅值为0.4v的噪声\r\n");
                // 511/4095*3.3 = 0.4v
                SetWaveGenerationAmplitude(DAC_Channel_1, DAC_LFSRUnmask_Bits8_0);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, ENABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, DISABLE);
                break;
            case 5:
                printf("TIM6触发DMA产生100Hz幅值为3.3V的正弦波\r\n");
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, DISABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, DISABLE);
                DAC_SetChannel1Data(DAC_Align_12b_R, 0);
                DAC_SetChannelTrigger(DAC_Channel_1, DAC_Trigger_T6_TRGO);
                DAC_DMACmd(DAC_Channel_1, ENABLE);
                DMA_Cmd(DMA2_Channel3, ENABLE);
                TIM_Cmd(TIM6, ENABLE);
                break;

            default:
                WaveType = 0;
                TIM_Cmd(TIM6, DISABLE);
                DAC_DMACmd(DAC_Channel_1, DISABLE);
                DMA_Cmd(DMA2_Channel3, DISABLE);
                printf("利用延时产生50Hz幅值为1.5V的正弦波\r\n");
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Noise, DISABLE);
                DAC_WaveGenerationCmd(DAC_Channel_1, DAC_Wave_Triangle, DISABLE);
                DAC_SetChannelTrigger(DAC_Channel_1, DAC_Trigger_None);
                //                DAC_SetChannel1Data(DAC_Align_12b_R, 0);
                break;
        }
        //         printf("WaveType: %d\r\n", WaveType);
    }
}

void DAC1_DMA_Init(void)
{
    DMA_InitTypeDef DMA_InitStructure = {0};
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA2, ENABLE);

    DMA_StructInit(&DMA_InitStructure);
    /* Note:DAC1--->DMA2.CH3   DAC2--->DMA2.CH4 */
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32) & (DAC->R12BDHR1);
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)DAC_Value;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = 64;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

    DMA_Init(DMA2_Channel3, &DMA_InitStructure);
    //    DMA_Cmd(DMA2_Channel3, ENABLE);
}

#define USE_SWITCH

int main(void)
{
    RCC_ClocksTypeDef clocks;
    GPIO_InitTypeDef gpio_init;

    Delay_Init();
    USART_Printf_Init(115200);
    SystemCoreClockUpdate();
    RCC_GetClocksFreq(&clocks);

    printf("系统时钟频率: %dHz\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟频率: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2总线时钟频率: %dHz\r\n", clocks.PCLK2_Frequency);
    DAC1_Init();
    PA0_INT_KEY_Init();
    TIM6_Frequency_Init(100);
    DAC1_DMA_Init();

    printf("将PA0跟KEY相连，示波器接PA4，通过按键切换不同的波形\r\n");

    gpio_init.GPIO_Pin = GPIO_Pin_1;
    gpio_init.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &gpio_init);

    printf("利用延时产生50Hz幅值为1.5V的正弦波\r\n");
    WaveType = 0;
    Delay_Ms(10);

    while (1) {

#ifdef USE_SWITCH
        switch (WaveType) {
            // 产生50Hz幅值为1.5V的正弦波
            case 0:
#else
        if (WaveType == 0) {
#endif
                for (int i = 0; i < sizeof(DAC_Value) / sizeof(DAC_Value[0]); i++) {
                    DAC_SetChannel1Data(DAC_Align_12b_R, (int)(DAC_Value[i] * 0.5));
                    // 1000000/50/64=312.5
                    Delay_Us(312);
                }
#ifdef USE_SWITCH
                break;
#else
        } else if (WaveType == 1) {
#endif
                // 利用内部波形发生器产生幅值为1.5V的三角波
#ifdef USE_SWITCH
            case 1:
#endif
                DAC_SetChannel1Data(DAC_Align_12b_R, 0);
                DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);
                Delay_Us(10);
#ifdef USE_SWITCH
                break;
#else
        } else if (WaveType == 2) {
#endif
                // 利用内部波形发生器产生幅值为1.5v的噪声波形
#ifdef USE_SWITCH
            case 2:
#endif
                DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);
                Delay_Us(10);
#ifdef USE_SWITCH
                break;
#else
        } else if (WaveType == 3 || WaveType == 4) {
#endif
                // 50Hz幅值为1.5v的正弦波叠加幅值为1.5v的三角波
#ifdef USE_SWITCH
            case 3:
            // 50Hz幅值为1.5v的正弦波叠加幅值为1.5v的噪声
            case 4:
#endif
                for (int y = 0; y < sizeof(DAC_Value) / sizeof(DAC_Value[0]); y++) {
                    DAC_SetChannel1Data(DAC_Align_12b_R, (uint16_t)(DAC_Value[y] * 0.5));
                    DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);
                    Delay_Us(312);
                }
#ifdef USE_SWITCH
                break;
#else
        }
#endif

#ifdef USE_SWITCH
            default:
                break;
        }
#endif
        // printf("%d,\r\n",DAC_GetDataOutputValue(DAC_Channel_1));
    }
}
