#include "debug.h"

int main(void)
{
    BitAction iwdg_state;

    // 配置中断优先级分组
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    // 系统延时函数初始化
    Delay_Init();
    // 调试打印串口初始化,usart1(PA.9)打印调试信息
    USART_Printf_Init(115200);
    // 根据用户配置重新计算并更新全局变量SystemCoreClock
    SystemCoreClockUpdate();
    printf("\r\n系统时钟:%d\r\n", SystemCoreClock);

    // 打印系统启动原因
    if (RCC_GetFlagStatus(RCC_FLAG_PORRST) == SET) {
        printf("系统上电启动\r\n");
    } else if (RCC_GetFlagStatus(RCC_FLAG_PINRST) == SET) {
        printf("NRST引脚复位\r\n");
    } else if (RCC_GetFlagStatus(RCC_FLAG_SFTRST) == SET) {
        printf("软件重启\r\n");
    } else if (RCC_GetFlagStatus(RCC_FLAG_WWDGRST) == SET) {
        printf("窗口看门狗复位\r\n");
    } else if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) == SET) {
        printf("独立看门狗复位\r\n");
    } else if (RCC_GetFlagStatus(RCC_FLAG_LPWRRST) == SET) {
        printf("低功耗复位\r\n");
    }

    // 上电模拟1次软件重启
    if (RCC_GetFlagStatus(RCC_FLAG_SFTRST) == RESET && RCC_GetFlagStatus(RCC_FLAG_PORRST) == SET) {
        printf("模拟软件重启...\r\n");
        RCC_ClearFlag();
        Delay_Ms(500);
        // 软件重启
        NVIC_SystemReset();
    }

    // PA.0初始化为带下拉输入
    GPIO_InitTypeDef gpio_init;
    gpio_init.GPIO_Mode = GPIO_Mode_IPD;
    gpio_init.GPIO_Pin = GPIO_Pin_0;
    GPIO_Init(GPIOA, &gpio_init);

    // 软件重启默认开启独立看门狗
    if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) == SET || RCC_GetFlagStatus(RCC_FLAG_SFTRST) == SET) {
        printf("启动独立看门狗,3.2s不喂狗就重启... \r\n");
        // 在操作独立看门狗寄存器前必须先往IWDG_CTLR中写入0x5555
        IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
        // 40KHz的LSI时钟64分频后给独立看门狗计数用 40K/32=125Hz
        IWDG_SetPrescaler(IWDG_Prescaler_32);
        // 配置为3.2s不喂狗的话就复位
        IWDG_SetReload(4000);
        // 启动看门狗，一但启动了就不能停止。只能复位后才能停
        IWDG_Enable();

        iwdg_state = ENABLE;
    }else{
        printf("复位后未启用独立看门狗!\r\n");
        iwdg_state = DISABLE;
    }

    RCC_ClearFlag();

    while (1) {
        if (iwdg_state) {
            if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0) == Bit_SET) {
                printf("模拟喂狗...\r");
                // 喂狗操作函数
                IWDG_ReloadCounter();
            } else {
                printf(".");
            }
            // 上面没有\n,若不人为刷新就会待到缓冲区满了后才会输出。程序会呈现卡在此处的效果
            fflush(stdout);
            Delay_Ms(1000);
        }
    }
}
