#include "debug.h"

u16 pbuf[] = {20, 20, 20, 20, 20, 20, 20, 20,
              20, 20, 20, 20, 20, 20, 20, 20,
              20, 20, 20, 20, 20, 20, 20, 60,
             };

void tim2_pwm_init(uint16_t arr, uint16_t psc, uint16_t ccp)
{
    GPIO_InitTypeDef gpio = {0};
    TIM_TimeBaseInitTypeDef timebase = {0};
    TIM_OCInitTypeDef oc = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

    gpio.GPIO_Pin = GPIO_Pin_0;
    gpio.GPIO_Mode = GPIO_Mode_AF_PP;
    gpio.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &gpio);

    timebase.TIM_Prescaler = psc;
    timebase.TIM_Period = arr;
    timebase.TIM_ClockDivision = TIM_CKD_DIV1;
    timebase.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM2, &timebase);

    oc.TIM_OCMode = TIM_OCMode_PWM1;
    oc.TIM_OutputState = TIM_OutputState_Enable;
    oc.TIM_Pulse = ccp;
    oc.TIM_OCPolarity = TIM_OCPolarity_High;
    TIM_OC1Init(TIM2, &oc);

    TIM_CtrlPWMOutputs(TIM2, ENABLE);
    TIM_OC1PolarityConfig(TIM2, TIM_OCPreload_Disable);
    TIM_ARRPreloadConfig(TIM2, ENABLE);

    // TIM_Cmd(TIM2, ENABLE);
}

void TIM2_DMA_Init(DMA_Channel_TypeDef *DMA_CHx, u32 ppadr, u32 memadr, u16 bufsize)
{
    DMA_InitTypeDef DMA_InitStructure = {0};

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    DMA_DeInit(DMA_CHx);
    DMA_InitStructure.DMA_PeripheralBaseAddr = ppadr;
    DMA_InitStructure.DMA_MemoryBaseAddr = memadr;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = bufsize;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA_CHx, &DMA_InitStructure);

    DMA_Cmd(DMA_CHx, ENABLE);
}

int main(void)
{
    RCC_ClocksTypeDef clocks;

    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    RCC_GetClocksFreq(&clocks);

    printf("系统时钟频率: %dHz\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟频率: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2总线时钟频率: %dHz\r\n", clocks.PCLK2_Frequency);

    printf("TIM1初始化\r\n");
//    280ns/1.12us 25% sw2812->0
//    tim2_pwm_init(80, 0, 20);
//    830ns/1.12us 75% sw2812->1
    tim2_pwm_init(80, 0, pbuf[0]);
    TIM2_DMA_Init(DMA1_Channel5, (u32)&TIM2->CH1CVR, (u32)pbuf, sizeof(pbuf) / sizeof(pbuf[0]));

    TIM_DMACmd(TIM2, TIM_DMA_Update, ENABLE);
    TIM_Cmd(TIM2, ENABLE);
    TIM_CtrlPWMOutputs(TIM2, ENABLE);

    while (1) {

    }

}
