---
title: CH32V30x 的CAN总线
categories: [rsic-v]
tags: [CAN]
author: TGL
date: 2023-09-11
---

## CAN 初始化

1. 开启引脚对应的 GPIO 模块时钟及复用时钟及 CAN 模块时钟
2. 配置 CAN_Tx 引脚为复用推挽输出，CAN_Rx 为浮空输入或上拉输入
3. 配置 CAN 模块的通信波特率及工作模式

| CAN 模块引脚 | 默认(00) | 重映射 1(01) | 重映射 2(10) | 重映射 3(11) |
| :----------: | :------: | :----------: | :----------: | :----------: |
|   CAN1_Rx    |   PA11   |     PB8      |     ---      |     PD0      |
|   CAN1_Tx    |   PA12   |     PB9      |     ---      |     PD1      |
|   CAN2_Rx    |   PB12   |     PB5      |     ---      |     ---      |
|   CAN2_Tx    |   PB13   |     PB6      |     ---      |     ---      |

| CAN 工作模式 | 工作模式描述                                           |
| ------------ | ------------------------------------------------------ |
| 正常模式     | 正常的收发                                             |
| 静默模式     | 发送内部直接环回到接收，接收外部发的数据，不向外发数据 |
| 环回模式     | 发送数据同时环回到接收，能向外发数据，不接收外部数据   |
| 静默环回模式 | 发送数据直接环回到接收，不向外发数据，不接收外部数据   |

<details>
<summary> 示例代码 </summary>

```c
static uint8_t CAN1_init(uint8_t brp, uint8_t bs1, uint8_t bs2, uint8_t sjw,
                        uint8_t mode)
{
    GPIO_InitTypeDef gpio;
    CAN_InitTypeDef CAN_InitSturcture;

    /// 开启 GPIOB 及功能复用时钟
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    /// 开启 CAN1 模块时钟
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    /// 将 CAN1 外设对就能的 PA11,PA12 重映射到 PB8,PB9上
    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    /// 将 CAN1_Rx 引脚 PA8 初始化为浮空输入或带上拉输入
    gpio.GPIO_Pin = GPIO_Pin_8;
    gpio.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &gpio);

    /// 将 CAN1_Tx 引脚 PA9 初始化为推挽复用输出
    gpio.GPIO_Pin = GPIO_Pin_9;
    gpio.GPIO_Mode = GPIO_Mode_AF_PP;
    gpio.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_Init(GPIOB, &gpio);

    /** CAN1 波特率及模式初始化
     *  CAN 波特率计算公式
     *  Bps = Fpclk1 / ((bs1+1+bs2+1+1) * brp )
     */
    CAN_StructInit(&CAN_InitSturcture);
    CAN_InitSturcture.CAN_BS1 = bs1;
    CAN_InitSturcture.CAN_BS2 = bs2;
    CAN_InitSturcture.CAN_SJW = sjw;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_InitSturcture.CAN_Mode = mode;

    return CAN_Init(CAN1, &CAN_InitSturcture);

}
```

</details>

## CAN 报文发送

1. 每个 CAN 模块有 3 个发送邮箱；
2. 在发送数据前先通过`TSTATR`寄存器中的`TMEx`位是否为 1，来判断哪个发送邮箱为空，`CAN_Transmit`函数也是根据该来选择发送邮箱的；
3. 将数据填入发送邮箱对应的`TXMIRx`,`TXMDTRx`,`TXMDLRx`,`TXMDHRx`寄存器中，并把`TXMIRx`的`TXRQ`位置位，当 CAN 总线进入空闲状态时就可以将该报文发送出去；
4. 当数据发送成功后，`TSTATR`寄存器中的`RQCPx`,`TXOKx`,`TMEx`全置 1，`CAN_TransmitStatus`函数也是根据这 3 位来判断是否发送成功的；
5. **注意**：如果开启了发送中，这时你再用`CAN_TransmitStatus`来判断是否发送成功就不可取了，因为在中断处理函数中调用的`CAN_ClearITPendingBit`已将`TSTATR`中的所有`RQCPx`位置 0 了；

## CAN 报文接收
