---
title: ICMP 协议应用
categories: Internet
tags: Netlib
date: 2023-08-11
author: 唐国林<tgl233@qq.com>
---

## ICMP 协议

### ICMP 分层结构

ICMP 与 IP 协议都位于网络层，但是 ICMP 在 TCP/IP 协议栈中的位置比 IP 协议略高一些，ICMP 报文需要封装在 IP 数据报的数据部分进行传输。ICMP 并不作为一个独立的层次，而只是作为 IP 层的一部分存在。

![Alt text](<NetLib 库初始化/TCPIP 分层.svg>)

### ICMP 功能

#### 1. 报告差错

#### 2. 查询

### ICMP 封装

![Alt text](<ICMP 协议/ICMP 封装.svg>)

1. 类型字段占 1 字节（8 位），用于指示 ICMP 报文的类型。例如，0 表示回送应答（Echo Reply）, 3 表示目的地不可达（Destination Unreachable）。[关于 ICMP 报文的具体类型可以在 IANA 网站查看](http://www.iana.org/assignments/icmp-parameters/)。除了 RFC 792 规定的报文类型外，其他一些 RFC 文档（如 RFC 950、RFC 1256）规定了一些新的类型。
2. 代码字段占 1 字节（8 位），提供关于报文类型的进一步信息。例如，对于报文类型 3 来说，代码 0 表示网络不可达，代码 1 表示网络不可达。
3. 校验和字段占 2 字节，提供整个 ICMP 协议报文的校验和。校验方法与 IP 数据报首部校验和算法相同。校验和以 16 位为单位进行计算，校验和的初始值为 0，用 1 的补码算法对 16 位小数据块进行求和，最后再对结果求补码，便得到了报文的校验和。与 IP 数据报首部校验和不同的地方是 ICMP 协议校验和是整个报文的校验和，另外使用校验和进行校验的设备不是中间的路由器，而是最终的目的地。
4. 首部其他部分为 4 字节，针对每一种报文类型都是特定的。大部分差错报告报文未用到这一部分，参数错报告报文用到其中的一个字节作指针，请求应答报文对利用这 4 字节匹配请求与应答报文。不使用时将不使用的部分填 0。后面在介绍各类 ICMP 报文将详细介绍。

#### 查询请求

![Alt text](<ICMP 协议/icmp 查询.svg>)

- 类型码为：8 表明是回送(请求)报文
- 代码固定为：0
- ICMP 首部其它部分 4 字节，被划分成了`标识符(2字节)`和`序列号(2字节)` 这两个字段值在协议中没有正式定义，标识符一般为发起请求进程的进程 ID，序列号在发送每一个回送（请求）报文时被增加。可供选择的数据字段包含一个消息，这个报文必须由接收方在回送应答报文中完全一样地重复着。

#### 查询回送

![Alt text](<ICMP 协议/icmp 回送.svg>)

回送跟请求的唯一区别就是类型码变为 0 了

### Wireshark 实例

![图 1](<ICMP 协议/4c445039348087d8b253aad8b6a028001d460be7e4125603d2248950590b51de.png>)
![图 2](<ICMP 协议/5624718d82a6ef2ca9a15f131e5d6f666e7c9a877448362ddceaa5e119348ca9.png>)

数据包按请求（Echo request）和应答（Echo reply）成对产生。标识符（Identifier）均为 0x0200（512），序列号（Sequence number）初始值为 256，每发送一个回送（请求）报文时再增加 256。

## IPRaw_PING 工程

1. NetLib 库初始化
2. 创建 RAW Socket
   ![图 3](<ICMP 协议/c6e2802e6cdc77c4d878eb28c464783fca011fdbe1f99dcdf518bc5c0fd99149.png>)  
   **在 RAW 模式下，`SourPort` 用于指定 IP 首部中的 `协议` 字段，ICMP 对应协议为：1**
3. 初始化 ICMP 数据内容
   ![图 6](<ICMP 协议/ecec791af22ff1ad0cf76b33a43ba20a56c1e2af47ed1748512193c0091ed164.png>)

### ICMP 校验和

ICMP 首部的校验和步骤：

- 首先是将<mark style="background:pink;">除校验和</mark>字段外的所有数据，按 16 位数进求和；
- 再将求得的和按 16 位数进行高低位相加；
- 再用求和数据的 16 位反码也就是(0xffff-和)；

下图是将数据段全置 0 的 ICMP 数据包，校验和为：

1. `0x0800 + 0x0000 + 0x0200 + 0x0065 = 0x0a65`
2. `0x0a65 + 0x0000 = 0x0a65`
3. `0xffff - 0x0a65 = 0xf59a`

![图 4](<ICMP 协议/d04e2642c1888f5189d190703a077457a6150d626d345f20653438d07263727d.png>)

1. `hex(0x0800 + 0x0200 + 0x0066 + 0x6162 + 0x6364 + 0x6566 + 0x6768 + 0x696a + 0x6b6c + 0x6d6e + 0x6f70 + 0x7172 + 0x7374 + 0x7576 + 0x7778 + 0x797a + 0x6162 + 0x6364 + 0x6566) = 0x6c328`
2. `hex(0x6 + 0x0c328) = 0xc32e`
3. `hex( 0xffff - 0xc32e) = 0x3cd1`

![图 5](<ICMP 协议/08daeee9e177cff7e252ea966cdb0eb6b49d1818c7253b1e95d2fa5950188a9a.png>)
